#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Caja Wipe - Extension for Caja to wipe files and/or free disk space.
# Copyright (C) 2016 Caja Wipe authors.
# Dual-licensed under the GPLv2 (or, at your option, any later version) and
# custom licensing terms:
# GPL v2:
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with
#   this program; if not, write to the Free Software Foundation, Inc., 51
#   Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Custom license terms:
#   You are hereby granted a perpetual, irrevocable license to copy, modify,
#   publish, release, and distribute this program as you see fit. However,
#   under no circumstances may you claim original authorship of this program;
#   you must credit the original author(s). You may not remove the listing of
#   the original author(s) from the source code, though you may change the
#   licensing terms. If you publish or release binaries or similarly compiled
#   files, you must credit the author(s) on your home and/or distribution page,
#   whichever applies. In your documentation, you must credit the author(s) for
#   the portions of their code you have used. This, of course, does not revoke
#   or change your right to claim original authorship to any portions of the
#   code that you have written.
#
#   You must agree to assume all liability for your use of the program, and to
#   indemnify and hold harmless the author(s) of this program from any liability
#   arising from use of this program, including, but not limited to: loss of
#   data, death, dismemberment, or injury, and all consequential and
#   inconsequential damages.
#
#   For clarification, contact Fred Barclay:
#       https://github.com/Fred-Barclay
#       BugsAteFred@gmail.com

import os
import gettext
import subprocess, gi

gi.require_version('Gtk', '3.0')
from gi.repository import Caja, GObject, GLib, Gtk

import threading
import gettext
# import locale
import gettext

t = gettext.translation('wipe_fm', "/usr/share/locale/")
t.install()
_ = t.gettext

class CajaWipe(GObject.GObject, Caja.MenuProvider):
    '''Simple Caja extension to wipe files, using the secure-delete suite.'''

    def __init__(self):
        GObject.Object.__init__(self)

    # Choosing the file/directory
    def get_file_items(self, window, files):
        '''Returns the menu items to display when one or more files/folders are
        selected.'''

        global filelist
        global pwd

        pwd = None
        filelist = []
        for file in files:
            if pwd == None:
                pwd = file.get_parent_location().get_path()
            name = file.get_name()
            filelist += [name]

        if pwd == None or len(filelist) == 0:
            return

        elif not os.access(pwd, os.W_OK):
            return

        if file.is_directory():
            text = _("Wipe these folders") if len(files) > 1 else _("Wipe this folder")
            item = Caja.MenuItem(name=f'Extra actions::{text}',
                                 label=text,
                                 tip='',
                                 icon='')
            item.connect('activate', self.wipe_file, file)
            return [item]
        else:
            text = _("Wipe these files") if len(files) > 1 else _("Wipe this file")
            item = Caja.MenuItem(name=f'Extra actions::{text}',
                                 label=text,
                                 tip='',
                                 icon='')
            item.connect('activate', self.wipe_file, file)
            return [item]

    def wipe_file(self, menu, file):
        win = InfoClass()
        response = win.wipe_ask_dialog(self)
        if response[0]:
            for filename in filelist:
                path = f"{pwd}/{filename}"
                if response[1] != "":
                    cmd = ["wipe", "-rfq", "-Q", response[1], path]
                else:
                    cmd = ["wipe", "-rfq", "-Q", "4", path]
                self.async_start(cmd)
            win.print_info(_("_The secure wipe process will take time and has been moved to the background"))

    def async_start(self, cmd):
        def thread_run():
            GLib.idle_add(cleanup, self.comd(cmd))

        def cleanup(s):
            try:
                t.join()
            except Exception as ex:
                win = InfoClass()
                win.print_error(ex)

        t = threading.Thread(None, thread_run)
        t.start()

    def comd(self, cmd):
        subprocess.call(cmd)


class InfoClass(Gtk.Window):
    def __init__(self):
        Gtk.Window.__init__(self)

    def print_info(self, info):
        dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, "")
        text = _("<b>Information</b>")
        dialog.set_markup(text)
        dialogBox = dialog.get_content_area()
        dialog.set_resizable(True)
        dialog.set_size_request(300, 60)
        dialogBox.set_spacing(10)
        label = Gtk.Label()
        label.set_text(f"   {info}    ")
        dialogBox.pack_start(label, True, True, 0)
        dialog.show_all()
        dialog.run()
        dialog.destroy()

    def print_error(self, error):
        dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR, Gtk.ButtonsType.CANCEL, "")
        text = _("<b>Error</b>")
        dialog.set_markup(text)
        dialogBox = dialog.get_content_area()
        dialog.set_resizable(True)
        dialog.set_size_request(300, 60)
        dialogBox.set_spacing(10)
        label = Gtk.Label()
        label.set_text(f"   {error}    ")
        dialogBox.pack_start(label, True, True, 0)
        dialog.show_all()
        dialog.run()
        dialog.destroy()

    def wipe_ask_dialog(self, widget):
        dialogWindow = Gtk.MessageDialog(parent=self,
                                         modal=True, destroy_with_parent=True,
                                         message_type=Gtk.MessageType.QUESTION,
                                         buttons=Gtk.ButtonsType.OK_CANCEL)
        text = _("Secure wipe")
        dialogWindow.set_title(text)
        dialogWindow.set_resizable(True)
        dialogBox = dialogWindow.get_content_area()
        label = Gtk.Label()
        text = _("Do you want to securely wipe your data?\nIt will be impossible to restore your data.")
        label.set_text(text)
        mainbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=2)
        mainbox.pack_start(label, False, False, 0)

        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=2)
        label = Gtk.Label()
        text = _("Specify the number of data overwriting cycles: ")
        label.set_text(text)
        box.pack_start(label, False, False, 0)
        entry = Gtk.Entry()
        entry.set_text("4")
        entry.connect('changed', self.numbify)

        box.pack_start(entry, False, False, 0)

        mainbox.pack_start(box, False, False, 0)

        dialogBox.pack_start(mainbox, False, False, 0)

        dialogWindow.set_border_width(3)
        dialogBox.set_spacing(2)
        dialogWindow.set_default_size(300, 50)
        dialogWindow.show_all()
        response = dialogWindow.run()
        if response == Gtk.ResponseType.OK:
            text = entry.get_text()
            dialogWindow.destroy()
            return [True, text]
        else:
            dialogWindow.destroy()
            return [False, ""]

    def numbify(self, entry):
        text = entry.get_text().strip()
        entry.set_text(''.join([i for i in text if i in '0123456789']))


