#!/usr/bin/python3

import os
import subprocess, gi, logging

gi.require_version('Gtk', '3.0')
from gi.repository import Caja, GObject, Gtk, GLib, Pango
from pathlib import Path

import threading
import gettext

t = gettext.translation('extras_fm', "/usr/share/locale/")
t.install()
_ = t.gettext

class ExtraExtensions(GObject.GObject, Caja.MenuProvider):
    '''Simple Caja extensions:
        pdf compression;
        add files to templates;
        get files path;
        mount iso;
        smb run button.'''

    def __init__(self):
        GObject.Object.__init__(self)

    # Choosing the file/directory
    def get_file_items(self, window, files):
        '''Returns the menu items to display when one or more files/folders are
        selected.'''

        global filelist
        global pwd

        pwd = None
        filelist = []
        for file in files:
            if pwd == None:
                pwd = file.get_parent_location().get_path()
            name = file.get_name()
            filelist += [name]

        # if len(files) == 1:
        #     file = files[0]
        #     pwd_mount = file.get_mount()
        #     if pwd_mount:
        #         if "smb://" in str(pwd_mount.get_root().get_uri()):
        #             text = _("Smb start button")
        #             item = Caja.MenuItem(
        #                 name='SmbRunButton::Smb_Run_Button',
        #                 label=text,
        #                 tip=text
        #             )
        #             item.connect('activate', self.smb_run_button, file)
        #             return [item]

        if pwd == None or len(filelist) == 0:
            return

        elif not os.access(pwd, os.W_OK):
            return

        text = _("Extra actions")
        top_menuitem = Caja.MenuItem(name=f'Extra actions::extras',
                                     label=text,
                                     tip='',
                                     icon='')
        extra_actions = Caja.Menu()
        top_menuitem.set_submenu(extra_actions)

        text = _("Add to templates")
        item = Caja.MenuItem(name=f'Extra actions::{text}',
                             label=text,
                             tip='',
                             icon='')
        extra_actions.append_item(item)
        item.connect('activate', self.copy_to_template, file)

        if len(filelist) == 1:
            text = _("Copy location")
            item = Caja.MenuItem(name=f'Extra actions::copylocation',
                                 label=text,
                                 tip='',
                                 icon='')
            extra_actions.append_item(item)
            item.connect('activate', self.pwd_file, file)

        if "pdf" in file.get_mime_type():
            text = _("PDF compression")
            item = Caja.MenuItem(
                name='CompressPdf::Compress_PDF',
                label=text,
                tip=text,
                icon=''
            )
            extra_actions.append_item(item)
            item.connect('activate', self.compress_pdf, file)



        if len(files) == 1 and "x-cd-image" in file.get_mime_type():
            text = _("Mount ISO")
            item = Caja.MenuItem(
                name='MountISO::Mount_ISO',
                label=text,
                tip=text,
                icon=''
            )
            extra_actions.append_item(item)
            item.connect('activate', self.mount_iso, file)

        return [top_menuitem]

    def pwd_file(self, menu, file):
        pwd = file.get_parent_location().get_path()
        name = file.get_name()
        path = pwd + "/" + name
        cmd = f"xsel -bips <<< '{path} ' "
        subprocess.call(cmd, shell=True)

    def copy_to_template(self, menu, file):
        win = InfoClass()
        text = _("Templates")
        dest = f"{Path.home()}/{text}/"
        for filename in filelist:
            path = pwd + "/" + filename
            if path == f"/{filename}":
                text = _("Attempt to copy to current directory")
                win.print_error(text)
            elif os.path.exists(f"{dest}/{filename}"):
                cmd = ["/usr/bin/cp", "-rf", path, dest]
                win = Async_gui(cmd, True)
                win.connect("destroy", Gtk.main_quit)
                win.show_all()
                Gtk.main()
            else:
                cmd = ["/usr/bin/cp", "-r", path, f"{dest}"]
                win = Async_gui(cmd, False)
                win.connect("destroy", Gtk.main_quit)
                win.show_all()
                Gtk.main()

    def mount_iso(self, menu, file):
        win = InfoClass()
        imageselect = pwd + "/" + str(filelist[0])
        wholename = filelist[0]
        name = wholename.split(".iso")[0]
        isoid = os.popen(
            f"isoinfo -ignore-error -d -i {imageselect} | grep 'Volume id:' | cut -c 12- | sed -n '$p'").readline().strip()
        isoidfind = os.popen(f"mount | grep \"{isoid}\" | wc -l").readline().strip()
        isodevfind = os.popen(f"mount | grep {name}  | wc -l").readline().strip()
        mountscheck = int(isoidfind) + int(isodevfind)
        if mountscheck == 0:
            response = win.choose_folder_dialog(self)
            if response[0]:
                response.append([filelist[0]])
                path = os.popen(f"mount | grep {response[1]}  | wc -l").readline().strip()
                if int(path) == 0:
                    self.async_start(response, "iso")
                else:
                    text = _("Select another folder")
                    win.print_error(text)
        else:
            text = _("ISO already mounted")
            win.print_error(text)

    def compress_pdf(self, menu, file):
        win = InfoClass()
        response = win.compress_ask_dialog(self, filelist)
        if response[0]:
            self.async_start(response, "pdf")
            text = _("A background process has started\nPlease wait for it to complete")
            win.print_info(text)

    def async_start(self, response, flag):
        def thread_run():
            GLib.idle_add(cleanup, self.comd(response, flag))

        def cleanup(s):
            try:
                t.join()
            except Exception as ex:
                win = InfoClass()
                win.print_error(ex)

        t = threading.Thread(None, thread_run)
        t.start()

    def comd(self, response, flag):
        if flag == "iso":
            os.system(f"pkexec mount -o loop {pwd}/{response[2][0]} {response[1]}")
        elif flag == "pdf":
            for elem in response[1]:
                path = f"{elem[2]}"
                text = _("changed")
                name = f'{elem[2].split(".pdf")[0]}_{text}.pdf'
                format_type = f"{elem[0]}"
                text0 = _("Normal")
                text1 = _("Worst")
                text2 = _("Medium")
                text3 = _("High")
                text4 = _("Best")
                if format_type == text0:
                    format_type = "/default"
                elif format_type == text1:
                    format_type = "/screen"
                elif format_type == text2:
                    format_type = "/ebook"
                elif format_type == text3:
                    format_type = "/printer"
                elif format_type == text4:
                    format_type = "/prepress"
                os.system(
                    f"gs -sDEVICE=pdfwrite -dPDFSETTINGS={format_type} -dSAFER -dNOPAUSE -dQUIET -dBATCH -sOutputFile='{name}' '{path}'")

    # def smb_run_button(self, menu, file):
    #     pwd = file.get_mount()
    #     if pwd is not None and pwd != "":
    #         text = _("on")
    #         pwd = pwd.get_name().split(f' {text} ')
    #         name = pwd[0]
    #         smb = pwd[1]
    #         btn = '#!/usr/bin/env xdg-open\n' \
    #               '[Desktop Entry]\n' \
    #               'Version=1.0\n' \
    #               'Type=Application\n' \
    #               'Terminal=false\n' \
    #               'Icon=folder\n' \
    #               'Icon[ru_RU]=folder\n' \
    #               f'Exec=caja smb://{smb}/{name}\n' \
    #               f'Name[ru_RU]={name}\n' \
    #               f'Name={name}\n'
    #         text = _("Desktop")
    #         cmd = ["touch", f"{Path.home()}/{text}/{name}.desktop"]
    #         subprocess.call(cmd)
    #         f = open(f"{Path.home()}/{text}/{name}.desktop", "w")
    #         f.write(btn)
    #         f.close()
    #         cmd = ["chmod", "+x", f"{Path.home()}/{text}/{name}.desktop"]
    #         subprocess.call(cmd)

class InfoClass(Gtk.Window):
    def __init__(self):
        Gtk.Window.__init__(self)

    def print_info(self, info):
        dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, "")
        dialog.set_markup(_("<b>Information</b>"))
        dialogBox = dialog.get_content_area()
        dialog.set_resizable(True)
        dialog.set_size_request(300, 60)
        dialogBox.set_spacing(10)
        label = Gtk.Label()
        label.set_text(f"   {info}    ")
        dialogBox.pack_start(label, True, True, 0)
        dialog.show_all()
        dialog.run()
        dialog.destroy()

    def print_error(self, error):
        dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR, Gtk.ButtonsType.CANCEL, "")
        text = _("<b>Error</b>")
        dialog.set_markup(text)
        dialogBox = dialog.get_content_area()
        dialog.set_resizable(True)
        dialog.set_size_request(300, 60)
        dialogBox.set_spacing(10)
        label = Gtk.Label()
        label.set_text(f"   {error}    ")
        dialogBox.pack_start(label, True, True, 0)
        dialog.show_all()
        dialog.run()
        dialog.destroy()

    def compress_ask_dialog(self, widget, filelist):
        dialogWindow = Gtk.MessageDialog(parent=self,
                                         modal=True, destroy_with_parent=True,
                                         message_type=Gtk.MessageType.QUESTION,
                                         buttons=Gtk.ButtonsType.OK_CANCEL)

        dialogWindow.set_title(_("Output file quality"))
        dialogWindow.set_resizable(True)
        dialogBox = dialogWindow.get_content_area()

        self.combo_type = Gtk.ComboBoxText()
        self.combo_liststore = Gtk.ListStore(str)
        self.liststore = Gtk.ListStore(str, str, str)
        text = _("Normal")
        self.combo_liststore.append([text])

        for file in filelist:
            self.liststore.append([text, f"{file}", f"{pwd}/{file}"])

        text = _("Worst")
        self.combo_liststore.append([text])

        text = _("Medium")
        self.combo_liststore.append([text])

        text = _("High")
        self.combo_liststore.append([text])

        text = _("Best")
        self.combo_liststore.append([text])

        treeview = Gtk.TreeView(model=self.liststore)

        renderer_combo = Gtk.CellRendererCombo()
        renderer_combo.set_property("editable", True)
        renderer_combo.set_property("model", self.combo_liststore)
        renderer_combo.set_property("text-column", 0)
        renderer_combo.set_property("has-entry", False)
        renderer_combo.connect("edited", self.on_combo_liststore_changed)

        text = _("Format")
        column_combo = Gtk.TreeViewColumn(text, renderer_combo, text=0)
        treeview.append_column(column_combo)

        renderer_text = Gtk.CellRendererText()
        text = _("File")
        column_text = Gtk.TreeViewColumn(text, renderer_text, text=1)
        treeview.append_column(column_text)
        dialogWindow.set_size_request(400, -1)
        dialogBox.pack_end(treeview, False, False, 0)
        dialogWindow.show_all()
        response = dialogWindow.run()
        dialogWindow.destroy()
        if (response == Gtk.ResponseType.CANCEL):
            return [False, None]
        else:
            return [True, self.liststore]

    def on_combo_liststore_changed(self, widget, path, text):
        self.liststore[path][0] = text

    def choose_folder_dialog(self, widget):
        text1 = _("Select directory(s)")
        text2 = _("Select directory")
        dialog = Gtk.FileChooserDialog(text1, self,
                                       Gtk.FileChooserAction.SELECT_FOLDER,
                                       (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                                        text2, Gtk.ResponseType.OK))

        dialog.set_default_size(800, 400)
        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            file_name = dialog.get_filename()
            dialog.destroy()
            return [True, file_name]
        else:
            dialog.destroy()
            return [False, ""]


class Async_gui(Gtk.ApplicationWindow):
    def __init__(self, cmd, exist):
        super(Async_gui, self).__init__()
        self.cmd = cmd
        self.exist = exist
        self.set_position(Gtk.WindowPosition.CENTER)

        self.set_default_size(300, 100)
        self.set_border_width(5)
        hb = Gtk.HeaderBar()
        hb.set_show_close_button(False)
        self.set_titlebar(hb)
        main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
        listbox = Gtk.ListBox()
        listbox.set_selection_mode(Gtk.SelectionMode.NONE)

        main_box.pack_start(listbox, True, True, 0)
        self.set_title("")
        ###############################################
        row = Gtk.ListBoxRow()
        hbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=15)
        self.label = Gtk.Label(_("The file exists. Replace?"))
        font = Pango.FontDescription("Tahoma 16")
        self.label.override_font(font)
        hbox.pack_start(self.label, True, True, 0)
        self.spinner = Gtk.Spinner()
        hbox.pack_end(self.spinner, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        ###############################################
        row = Gtk.ListBoxRow()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=15)
        self.button_update = Gtk.Button.new_with_label(_("Yes"))
        self.button_update.connect("clicked", self.button_update_clicked)
        hbox.pack_start(self.button_update, True, True, 0)

        self.button_close = Gtk.Button.new_with_label(_("Close"))
        self.button_close.connect("clicked", self.button_close_clicked)
        hbox.pack_start(self.button_close, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        ###############################################
        self.add(main_box)
        if not self.exist:
            self.call_async()

    def button_update_clicked(self, button):
        self.call_async()

    def call_async(self):
        self.spinner.start()
        self.label.set_text(_("A background process has started\nPlease wait for it to complete"))
        self.set_position(Gtk.WindowPosition.CENTER)
        self.button_update.set_sensitive(False)
        self.button_close.set_sensitive(False)

        def thread_run():
            GLib.idle_add(cleanup, self.comd())

        def cleanup(s):
            try:
                t.join()
                self.button_close.set_sensitive(True)
                self.spinner.stop()
                self.label.set_text(_("Operation completed"))
                self.resize(300, 120)
                self.destroy()
            except Exception as ex:
                self.button_close.set_sensitive(True)
                self.spinner.stop()
                text = _("Something went wrong:")
                self.label.set_text(f"{text}{ex}")
                self.resize(300, 120)

        t = threading.Thread(None, thread_run)
        t.start()

    def comd(self):
        subprocess.call(self.cmd)

    def button_close_clicked(self, button):
        self.destroy()