import hashlib, os, threading, subprocess, re, time
from pathlib import Path
from gi.repository import Caja, Gtk, GObject, GLib

import gettext

t = gettext.translation('gost_fm', "/usr/share/locale/")
t.install()
_ = t.gettext

GOST_FOLDER = f"{Path.home()}/.gost/"

def gost_is_enabled():
    grep_config = os.popen("grep -rn '@SECLEVEL=1:aGOST:aGOST01:kGOST:GOST94:GOST89MAC:kEECDH:kRSA:kEDH:kPSK:kDHEPSK"
                           ":kECDHEPSK:-aDSS:-3DES:!DES:!RC4:!RC2:!IDEA:-SEED:!eNULL:!aNULL:!MD5:-SHA384:-CAMELLIA"
                           ":-ARIA:-AESCCM8' /etc/crypto-policies/back-ends/openssl.config").readlines()
    if grep_config:
        return True
    return False


class MD5SumPropertyPage(GObject.GObject, Caja.PropertyPageProvider):
    def __init__(self):
        GObject.Object.__init__(self)

    def get_property_pages(self, files):
        if len(files) != 1:
            return

        file = files[0]
        if file.get_uri_scheme() != 'file':
            return
    
        if file.is_directory():
            return

        self.filename = file.get_location().get_path()

        self.property_label = Gtk.Label(_("Check sum"))
        self.property_label.show()

        self.main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=5)
        self.main_box.show()

        listbox = Gtk.ListBox()
        listbox.set_selection_mode(Gtk.SelectionMode.NONE)
        self.main_box.pack_start(listbox, True, True, 0)

        listbox.show()

        #################################
        row = Gtk.ListBoxRow()
        row.show()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)
        hbox.set_homogeneous(True)
        hbox.show()
        self.md5label = Gtk.Entry()
        self.md5label.set_property("editable", False)
        self.md5label.show()
        hbox.pack_start(self.md5label, True, True, 0)
        button_md5 = Gtk.Button.new_with_label(_("Calculate MD5"))
        button_md5.connect("clicked", self.on_button_md5_click)
        button_md5.set_size_request(100, 30)
        button_md5.show()
        hbox.pack_start(button_md5, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        #################################
        row = Gtk.ListBoxRow()
        row.show()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)
        hbox.set_homogeneous(True)
        hbox.show()
        self.gostlabel = Gtk.Entry()
        self.gostlabel.set_property("editable", False)
        self.gostlabel.show()
        hbox.pack_start(self.gostlabel, True, True, 0)

        self.button_gost = Gtk.Button.new_with_label(label=_("Calculate GOST R 34.11-2012"))
        self.button_gost .set_size_request(100, 30)

        self.button_gost.connect("clicked", self.on_button_gost_click)
        self.button_gost.show()
        hbox.pack_start(self.button_gost, True, True, 0)
        row.add(hbox)
        listbox.add(row)

        #################################
        row = Gtk.ListBoxRow()
        row.show()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)
        hbox.set_homogeneous(True)

        hbox.show()
        self.sha1label = Gtk.Entry()
        self.sha1label.set_property("editable", False)
        self.sha1label.show()
        hbox.pack_start(self.sha1label, True, True, 0)

        button_sha1 = Gtk.Button.new_with_label(_("Calculate SHA1"))
        button_sha1.connect("clicked", self.on_button_sha1_click)
        button_sha1.show()
        hbox.pack_start(button_sha1, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        #################################
        row = Gtk.ListBoxRow()
        row.show()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)
        hbox.set_homogeneous(True)
        hbox.show()
        self.sha256label = Gtk.Entry()
        self.sha256label.set_property("editable", False)
        self.sha256label.show()
        hbox.pack_start(self.sha256label, True, True, 0)

        button_sha256 = Gtk.Button.new_with_label(_("Calculate SHA256"))
        button_sha256.connect("clicked", self.on_button_sha256_click)
        button_sha256.show()
        hbox.pack_start(button_sha256, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        #################################
        #################################
        row = Gtk.ListBoxRow()
        row.show()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)
        hbox.set_homogeneous(True)

        hbox.show()
        self.csentry = Gtk.Entry()
        self.csentry.show()
        hbox.pack_start(self.csentry, True, True, 0)

        self.button_cs = Gtk.Button.new_with_label(_("Compare CS"))
        self.button_cs.set_sensitive(False)
        self.button_cs.connect("clicked", self.check_cs)
        self.button_cs.show()
        hbox.pack_start(self.button_cs, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        #################################
        #################################
        row = Gtk.ListBoxRow()
        row.show()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)
        hbox.show()
        self.label_result = Gtk.Label()
        self.label_result.set_markup(_("<b>Result: </b>"))
        self.label_result.show()
        hbox.pack_start(self.label_result, False, True, 0)
        row.add(hbox)
        listbox.add(row)
        #################################
        #################################

        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)
        hbox.show()
        self.label_gost_enable = Gtk.Label()
        self.label_gost_enable.set_markup(_("<b>Activate GOST</b>"))
        self.label_gost_enable.show()
        hbox.pack_start(self.label_gost_enable, False, True, 0)

        switch = Gtk.Switch()
        switch.show()

        switch.set_active(False)
        hbox.pack_start(switch, False, True, 5)
        # hbox.pack_start(switch, True, True, 0)
        row = Gtk.ListBoxRow()
        row.show()
        if gost_is_enabled():
            self.button_gost.set_sensitive(True)
            self.gostlabel.set_sensitive(True)
            switch.set_active(True)
        else:
            switch.set_active(False)
            self.gostlabel.set_sensitive(False)
            self.button_gost.set_sensitive(False)

        self.switcher_id_action = switch.connect("notify::active", self.on_switch_activated)

        row.add(hbox)
        listbox.add(row)

        return Caja.PropertyPage(name="CajaPython::md5_sum",
                                 label=self.property_label,
                                 page=self.main_box),

    def on_switch_activated(self, switch, gparam):
        if switch.get_active():
            switch.set_sensitive(False)
            os.system("pkexec /usr/bin/openssl-switch-config gost")
            time.sleep(2)
            if gost_is_enabled():
                switch.disconnect(self.switcher_id_action)
                switch.set_active(True)
                self.switcher_id_action = switch.connect("notify::active", self.on_switch_activated)
                self.button_gost.set_sensitive(True)
                self.gostlabel.set_sensitive(True)
                switch.set_sensitive(True)
            else:
                switch.disconnect(self.switcher_id_action)
                switch.set_active(False)
                self.switcher_id_action = switch.connect("notify::active", self.on_switch_activated)
                switch.set_sensitive(True)
        else:
            switch.set_sensitive(False)
            os.system("pkexec /usr/bin/openssl-switch-config default")
            time.sleep(2)
            if not gost_is_enabled():
                switch.disconnect(self.switcher_id_action)
                switch.set_active(False)
                self.switcher_id_action = switch.connect("notify::active", self.on_switch_activated)
                self.gostlabel.set_sensitive(False)
                self.button_gost.set_sensitive(False)
                switch.set_sensitive(True)
            else:
                switch.disconnect(self.switcher_id_action)
                switch.set_active(True)
                self.switcher_id_action = switch.connect("notify::active", self.on_switch_activated)
                switch.set_sensitive(True)


    def on_button_md5_click(self, button):
        md5sum = hashlib.md5()
        with open(self.filename, 'rb') as f:
            for chunk in iter(lambda: f.read(8192), b''):
                md5sum.update(chunk)
        f.close()

        self.md5label.set_text(md5sum.hexdigest())
        self.md5label.show()
        self.button_cs.set_sensitive(True)


    def on_button_gost_click(self, button):
        gost = os.popen(f"openssl dgst -engine gost -md_gost12_512 '{self.filename}'").readlines()[0].split("= ")[
            1].strip()
        self.gostlabel.set_text(gost)
        self.gostlabel.show()
        self.button_cs.set_sensitive(True)


    def on_button_sha1_click(self, button):
        sha1 = hashlib.sha1()
        with open(self.filename, 'rb') as f:
            for chunk in iter(lambda: f.read(8192), b''):
                sha1.update(chunk)
        f.close()

        self.sha1label.set_text(sha1.hexdigest())
        self.sha1label.show()
        self.button_cs.set_sensitive(True)


    def on_button_sha256_click(self, button):
        sha256 = hashlib.sha256()
        with open(self.filename, 'rb') as f:
            for chunk in iter(lambda: f.read(8192), b''):
                sha256.update(chunk)
        f.close()
        self.sha256 = sha256.hexdigest()
        self.sha256label.set_text(sha256.hexdigest())
        self.sha256label.show()
        self.button_cs.set_sensitive(True)


    def check_cs(self, button):
        cs = self.csentry.get_text()
        if cs != "":
            md5 = self.md5label.get_text()
            sha1 = self.sha1label.get_text()
            sha256 = self.sha256label.get_text()
            gost = self.gostlabel.get_text()
            if md5 == "" and sha1 == "" and sha256 == "" and gost == "":
                text = _("<b>Result: checksums are empty</b>")
                self.label_result.set_markup(text)
            elif md5 != "" and md5 == cs:
                text = _("<b>Result: checksum matches</b>")
                self.label_result.set_markup(text)
            elif sha1 != "" and sha1 == cs:
                text = _("<b>Result: checksum matches</b>")
                self.label_result.set_markup(text)
            elif sha256 != "" and sha256 == cs:
                text = _("<b>Result: checksum matches</b>")
                self.label_result.set_markup(text)
            elif gost != "" and gost == cs:
                text = _("<b>Result: checksum matches</b>")
                self.label_result.set_markup(text)
            else:
                text = _("<b>Result: checksum not matches</b>")
                self.label_result.set_markup(text)
        else:
            text = _("<b>Result: checksums are empty</b>")
            self.label_result.set_markup(text)


class GOSTCryptFiles(GObject.GObject, Caja.MenuProvider):
    def __init__(self):
        GObject.Object.__init__(self)

    def get_file_items(self, window, files):
        # if len(files) != 1:
        #     return

        global filelist
        global pwd
        pwd = None
        filelist = []
        for file in files:
            if pwd == None:  # first file: find path to directory
                pwd = file.get_parent_location().get_path()
            name = file.get_name()
            filelist += [name]

        if pwd == None or len(filelist) == 0:
            return

        # Make sure that the user has write permissions in this directory
        elif not os.access(pwd, os.W_OK):
            return
        if not gost_is_enabled():
            return
        # action for file(s) and directory(ies)
        text = _("GOST encryption")
        top_menuitem = Caja.MenuItem(name=f'GOSTCryptFiles::{text}',
                                     label=text,
                                     tip='',
                                     icon='')

        gost_actions = Caja.Menu()
        top_menuitem.set_submenu(gost_actions)
        Dont_hide_flag = True
        for file in filelist:
            if ".enc" in file:
                Dont_hide_flag = False
        if Dont_hide_flag:
            text = _("Secure Encryption")
            encrypt = Caja.MenuItem(name=f'GOSTCryptFiles::{text}',
                                    label=text,
                                    tip='',
                                    icon='')
            gost_actions.append_item(encrypt)
            encrypt.connect('activate', self.encrypt, file)
        Dont_hide_flag = True
        for file in filelist:
            if not ".enc" in file:
                Dont_hide_flag = False
        if Dont_hide_flag:
            text = _("Secure Decryption")
            decrypt = Caja.MenuItem(name=f'GOSTCryptFiles::{text}',
                                    label=text,
                                    tip='',
                                    icon='')
            gost_actions.append_item(decrypt)
            decrypt.connect('activate', self.decrypt, file)

        return [top_menuitem]

    def encrypt(self, menu, file):
        try:
            global GOST_FOLDER
            if not os.path.exists(f"{GOST_FOLDER}cert.pem"):
                win = CertCreate()
                win.connect("destroy", Gtk.main_quit)
                win.show_all()
                Gtk.main()
                if os.path.exists(f"{GOST_FOLDER}cert.pem"):
                    win = InfoClass()
                    for filename in filelist:
                        path = pwd + "/" + filename
                        input = path
                        output = f"{path}.enc"
                        if os.path.isdir(path):
                            dirname = os.path.basename(path)
                            output = f"{path}.tar.enc"
                            comd = [
                                ["tar", "-C", f"{pwd}", "-cvf", f"{path}.tar", f"{dirname}"],
                                ["openssl", "smime", "-encrypt", "-engine", "gost", "-gost89", "-binary", "-outform", "PEM",
                                 "-in", f"{path}.tar", "-out", output, f"{GOST_FOLDER}cert.pem"],
                                ["rm", "-rf", f"{input}"],
                                ["rm", "-rf", f"{input}.tar"]
                            ]
                            self.async_start(comd, True)
                        else:
                            comd = [
                                ["openssl", "smime", "-encrypt", "-engine", "gost", "-gost89", "-binary", "-outform", "PEM",
                                 "-in", input, "-out", output, f"{GOST_FOLDER}cert.pem"],
                                ["rm", "-rf", f"{input}"]]
                            self.async_start(comd, False)
                    win.print_info(_("The secure encryption process will take time and has been moved to the background"))
            else:
                win = InfoClass()
                for filename in filelist:
                    path = pwd + "/" + filename
                    input = path
                    output = f"{path}.enc"
                    if os.path.isdir(path):
                        dirname = os.path.basename(path)
                        output = f"{path}.tar.enc"
                        comd = [
                            ["tar", "-C", f"{pwd}", "-cvf", f"{path}.tar", f"{dirname}"],
                            ["openssl", "smime", "-encrypt", "-engine", "gost", "-gost89", "-binary", "-outform", "PEM",
                             "-in", f"{path}.tar", "-out", output, f"{GOST_FOLDER}cert.pem"],
                            ["rm", "-rf", f"{input}"],
                            ["rm", "-rf", f"{input}.tar"]
                        ]
                        self.async_start(comd, True)
                    else:
                        comd = [
                            ["openssl", "smime", "-encrypt", "-engine", "gost", "-gost89", "-binary", "-outform", "PEM",
                             "-in", input, "-out", output, f"{GOST_FOLDER}cert.pem"],
                            ["rm", "-rf", f"{input}"]]
                        self.async_start(comd, False)
                text = _("The secure encryption process will take time and has been moved to the background")
                win.print_info(text)
        except Exception as error:
            win = InfoClass()
            win.print_error(error)

    def decrypt(self, menu, file):
        try:
            global GOST_FOLDER
            if not os.path.exists(f"{GOST_FOLDER}key.pem"):
                win = InfoClass()
                text = _("Encryption certificate not found, try re-encrypting the original file")
                win.print_error(text)
            else:
                win = InfoClass()
                for filename in filelist:
                    path = pwd + "/" + filename
                    input = path
                    output = path[:-4]
                    if "tar" in output:
                        comd = [["openssl", "smime", "-engine", "gost", "-decrypt", "-in", f"{input}", "-inform", "PEM",
                                 "-inkey", f"{GOST_FOLDER}key.pem", "-out", output],
                                ["tar", "-C", f"{pwd}", "-xf", output],
                                ["echo"],
                                ["echo"]
                                ]
                        self.async_start(comd, True)
                    else:
                        comd = [["openssl", "smime", "-engine", "gost", "-decrypt", "-in", f"{input}", "-inform", "PEM",
                                 "-inkey", f"{GOST_FOLDER}key.pem", "-out", output],
                                ["echo"]
                                ]
                        self.async_start(comd, False)
                text = _("The secure decryption process will take time and has been moved to the background")
                win.print_info(text)
        except Exception as error:
            win = InfoClass()
            win.print_error(error)

    def async_start(self, cmd, isdir):
        def thread_run():
            if isdir:
                GLib.idle_add(cleanup, self.comd_dir(cmd))
            else:
                GLib.idle_add(cleanup, self.comd(cmd))

        def cleanup(s):
            try:
                t.join()
            except Exception as ex:
                win = InfoClass()
                win.print_error(ex)

        t = threading.Thread(None, thread_run)
        t.start()

    def comd(self, cmd):
        subprocess.call(cmd[0])
        subprocess.call(cmd[1])

    def comd_dir(self, cmd):
        subprocess.call(cmd[0])
        subprocess.call(cmd[1])
        subprocess.call(cmd[2])
        subprocess.call(cmd[3])


class CertCreate(Gtk.Window):
    def __init__(self):
        super(CertCreate, self).__init__()
        self.set_position(Gtk.WindowPosition.CENTER)
        self.set_default_size(500, 100)
        self.set_border_width(5)
        hb = Gtk.HeaderBar()
        hb.set_show_close_button(False)
        self.set_titlebar(hb)
        main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
        listbox = Gtk.ListBox()
        listbox.set_selection_mode(Gtk.SelectionMode.NONE)
        main_box.pack_start(listbox, True, True, 0)
        text = _("Initial creation of a signing certificate")
        self.set_title(text)
        ###############################################
        row = Gtk.ListBoxRow()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=15)
        text = _("Country name - 2 letters (example [RU])")
        label = Gtk.Label(text)
        self.country_name_entry = Gtk.Entry()
        hbox.pack_start(label, False, True, 0)
        hbox.pack_start(self.country_name_entry, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        ###############################################
        row = Gtk.ListBoxRow()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=15)
        text = _("Name of the area or region")
        label = Gtk.Label(text)
        self.state_name_entry = Gtk.Entry()
        hbox.pack_start(label, False, True, 0)
        hbox.pack_start(self.state_name_entry, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        ###############################################
        row = Gtk.ListBoxRow()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=15)
        text = _("Locality name")
        label = Gtk.Label(text)
        self.locality_name_entry = Gtk.Entry()
        hbox.pack_start(label, False, True, 0)
        hbox.pack_start(self.locality_name_entry, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        ###############################################
        row = Gtk.ListBoxRow()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=15)
        text = _("Name of the organization")
        label = Gtk.Label(text)
        self.org_name_entry = Gtk.Entry()
        hbox.pack_start(label, False, True, 0)
        hbox.pack_start(self.org_name_entry, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        ###############################################
        row = Gtk.ListBoxRow()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=15)
        text = _("Organization unit name")
        label = Gtk.Label(text)
        self.unit_name_entry = Gtk.Entry()
        hbox.pack_start(label, False, True, 0)
        hbox.pack_start(self.unit_name_entry, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        ###############################################
        row = Gtk.ListBoxRow()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=15)
        text = _("Your name or PC name")
        label = Gtk.Label(text)
        self.name_or_hostname_entry = Gtk.Entry()
        hbox.pack_start(label, False, True, 0)
        hbox.pack_start(self.name_or_hostname_entry, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        ###############################################
        row = Gtk.ListBoxRow()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=15)
        text = _("E-mail address")
        label = Gtk.Label(text)
        self.email_entry = Gtk.Entry()
        hbox.pack_start(label, False, True, 0)
        hbox.pack_start(self.email_entry, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        ###############################################
        row = Gtk.ListBoxRow()
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=15)
        text =_("Create")
        self.button_apply = Gtk.Button.new_with_label(text)
        self.button_apply.connect("clicked", self.button_apply_clicked)
        hbox.pack_start(self.button_apply, True, True, 0)
        text = _("Close")
        self.button_close = Gtk.Button.new_with_label(text)
        self.button_close.connect("clicked", self.button_close_clicked)
        hbox.pack_start(self.button_close, True, True, 0)
        row.add(hbox)
        listbox.add(row)
        ###############################################
        self.add(main_box)

    def button_apply_clicked(self, button):
        global GOST_FOLDER
        C = self.country_name_entry.get_text()
        ST = self.state_name_entry.get_text()
        L = self.locality_name_entry.get_text()
        O = self.org_name_entry.get_text()
        OU = self.unit_name_entry.get_text()
        emailAddress = self.email_entry.get_text()
        CN = self.name_or_hostname_entry.get_text()
        if C != "" and ST != "" and L != "" and O != "" and OU != "" and emailAddress != "" and CN != "":
            if self.check_patterns(C, emailAddress):
                try:
                    os.system(f"mkdir -p {GOST_FOLDER}")
                    os.system(f"openssl req -x509 -newkey gost2012_256 -pkeyopt paramset:A -nodes -keyout "
                              f"{GOST_FOLDER}key.pem -subj '/C={C}/ST={ST}/L={L}/O={O}/OU={OU}/"
                              f"emailAddress={emailAddress}/CN={CN}' -out {GOST_FOLDER}cert.pem -md_gost12_256 -utf8")
                    self.destroy()
                except Exception as error:
                    raise error
            else:
                info = InfoClass()
                text = _("English character input is supported, please check your input")
                info.print_error(text)

    def check_patterns(self, C, email):
        pattern = re.compile("^[a-zA-Z@.\-\'\" ]+$")
        if re.search(pattern, email):
            pattern = re.compile("^[a-zA-Z]+$")
            if re.search(pattern, C) and len(C) == 2:
                return True
        return False

    def button_close_clicked(self, button):
        self.destroy()

class InfoClass(Gtk.Window):
    def __init__(self):
        Gtk.Window.__init__(self)
        box = Gtk.Box()
        self.add(box)

    def print_error(self, error):
        dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR, Gtk.ButtonsType.CANCEL, "")
        text = _("<b>Error</b>")
        dialog.set_markup(text)
        dialogBox = dialog.get_content_area()
        dialog.set_resizable(True)
        dialog.set_size_request(300, 60)
        dialogBox.set_spacing(10)
        label = Gtk.Label()
        label.set_text(f"   {error}    ")
        dialogBox.pack_start(label, True, True, 0)
        dialog.show_all()
        dialog.run()
        dialog.destroy()

    def print_info(self, info):
        dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, "")
        text = _("<b>Information</b>")
        dialog.set_markup(text)
        dialogBox = dialog.get_content_area()
        dialog.set_resizable(True)
        dialog.set_size_request(300, 60)
        dialogBox.set_spacing(10)
        label = Gtk.Label()
        label.set_text(f"   {info}    ")
        dialogBox.pack_start(label, True, True, 0)
        dialog.show_all()
        dialog.run()
        dialog.destroy()

    def ask_about_source_file(self, widget, file_type):
        text1 = _("\nDelete")
        text2 = _("file(s)")
        dialogWindow = Gtk.MessageDialog(parent=self,
                                         modal=True, destroy_with_parent=True,
                                         message_type=Gtk.MessageType.QUESTION,
                                         buttons=Gtk.ButtonsType.NONE,
                                         text=f"{text1} {file_type} {text2}?")
        dialogWindow.set_title(_("Question"))
        text1 = _("Yes")
        text2 = _("No")
        dialogWindow.add_buttons(text1, Gtk.ResponseType.OK, text2, Gtk.ResponseType.CANCEL)
        dialogWindow.show_all()
        response = dialogWindow.run()
        dialogWindow.destroy()
        return response
