import os
import subprocess, gi
import threading

gi.require_version('Gtk', '3.0')
from gi.repository import Caja, GObject, GLib, Gtk

import gettext

t = gettext.translation('print_fm', "/usr/share/locale/")
t.install()
_ = t.gettext

SUPPORTED_FORMATS = 'image', 'text'

class CajaPrintFile(GObject.GObject, Caja.MenuProvider):
    def __init__(self):
        GObject.Object.__init__(self)

    # Choosing the file/directory
    def get_file_items(self, window, files):

        global filelist
        global pwd

        pwd = None
        filelist = []
        for file in files:
            if pwd == None:
                pwd = file.get_parent_location().get_path()
            name = file.get_name()
            filelist += [name]

        default_printer = os.popen("LANG=C lpstat -d").readlines()[0].strip()

        if default_printer == "no system default destination":
            return

        if pwd == None or len(filelist) == 0:
            return
        if not os.access(pwd, os.W_OK):
            return

        if file.is_directory():
            return
        else:
            mime_flag = False
            for mimetype in SUPPORTED_FORMATS:
                if mimetype in file.get_mime_type() or "doc" in file.get_mime_type() or "pdf" in file.get_mime_type():
                    if "x-cd-image" not in file.get_mime_type():
                        mime_flag = True
            if not mime_flag:
                return
            if not os.path.exists("/usr/bin/soffice"):
                return

            text = _("Print these files") if len(files) > 1 else _("Print this file")
            item = Caja.MenuItem(
                name='CajaPrintFile::Print_File',
                label=text,
                tip=text,
                        icon=""
            )
            item.connect('activate', self.print_file, file)
            return [item]

    def print_file(self, menu, file):
        win = InfoClass()
        if win.printer_ask_dialog(self):
            path = []
            for filename in filelist:
                path.append(f"{pwd}/{filename}")
            if len(path) != 0:
                cmd = ["/usr/bin/soffice", "-p"]
                for p in path:
                    cmd.append(p)
                # cmd = ["/usr/bin/soffice", "-p", " ".join(p for p in path)]
                # print(cmd)
                self.async_start(cmd)
                text = _("Job sent for printing")
                win.print_info(text)

    def async_start(self, cmd):
        def thread_run():
            GLib.idle_add(cleanup, self.comd(cmd))

        def cleanup(s):
            try:
                t.join()
            except Exception as ex:
                win = InfoClass()
                win.print_error(ex)

        t = threading.Thread(None, thread_run)
        t.start()

    def comd(self, cmd):
        subprocess.call(cmd)


class InfoClass(Gtk.Window):
    def __init__(self):
        Gtk.Window.__init__(self)

    def printer_ask_dialog(self, widget):
        dialogWindow = Gtk.MessageDialog(parent=self,
                                         modal=True, destroy_with_parent=True,
                                         message_type=Gtk.MessageType.QUESTION,
                                         buttons=Gtk.ButtonsType.OK_CANCEL)
        text = _("Printing")
        dialogWindow.set_title(text)
        dialogWindow.set_resizable(True)
        dialogBox = dialogWindow.get_content_area()
        label = Gtk.Label()
        text = _("Print to a printer with default settings?")
        label.set_text(text)
        dialogBox.pack_start(label, False, False, 0)
        dialogWindow.set_border_width(5)
        # dialogBox.set_spacing(10)
        dialogWindow.set_default_size(300, 50)
        dialogWindow.show_all()
        response = dialogWindow.run()
        if response == Gtk.ResponseType.OK:
            dialogWindow.destroy()
            return True
        else:
            dialogWindow.destroy()
            return False

    def print_info(self, info):
        dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, "")
        text = _("<b>Information</b>")
        dialog.set_markup(text)
        dialogBox = dialog.get_content_area()
        dialog.set_resizable(True)
        dialog.set_size_request(300, 60)
        dialogBox.set_spacing(10)
        label = Gtk.Label()
        label.set_text(f"   {info}    ")
        dialogBox.pack_start(label, True, True, 0)
        dialog.show_all()
        dialog.run()
        dialog.destroy()

    def print_error(self, error):
        dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR, Gtk.ButtonsType.CANCEL, "")
        text = _("<b>Error</b>")
        dialog.set_markup(text)
        dialogBox = dialog.get_content_area()
        dialog.set_resizable(True)
        dialog.set_size_request(300, 60)
        dialogBox.set_spacing(10)
        label = Gtk.Label()
        label.set_text(f"   {error}    ")
        dialogBox.pack_start(label, True, True, 0)
        dialog.show_all()
        dialog.run()
        dialog.destroy()